STATUS = "Status: {}"


###########################################
#####   AOI selector              #########
###########################################
FILE_PATTERN = 'aoi_{0}'
NO_SELECTION = 'No selection method have bee picked up'
NO_COUNTRY = 'No Country have been selected'
ASSET_ALREADY_EXIST = "The asset was already existing you can continue to use it. It's also available at :{0}"
ASSET_CREATED = "The asset has been created under the name : {0}"
NAME_USED = "The name was already in used, change it or delete the previous asset in your GEE acount"
NO_ASSET = "No Asset have been provided"
CHECK_IF_ASSET = "Check carefully that your string is an assetId"
NOT_AVAILABLE = "This function is not yet available"
NO_SHAPE = "No shape have been drawn on the map"
ERROR_OCCURED = "An error occured with provided .shp file"
AOI_MESSAGE='click on "selet these inputs" to validate your AOI'